Package: supersigs
Title: Supervised mutational signatures
Version: 1.19.0
Date: 2021-12-02
Authors@R: c(person(given = "Albert", 
                    family = "Kuo",
                    role = c("aut", "cre"),
                    email = "albertkuo@jhu.edu", 
                    comment = c("ORCID" = "0000-0001-5155-0748")),
             person(given = "Yifan", 
                    family = "Zhang",
                    role = c("aut")),
             person(given = "Bahman", 
                    family = "Afsari",
                    role = c("aut")),
             person(given = "Cristian", 
                    family = "Tomasetti",
                    role = c("aut"))) 
Depends: R (>= 4.1)
Imports: assertthat, caret, dplyr, tidyr, rsample, methods, rlang,
        utils, Biostrings, stats, SummarizedExperiment
Suggests: BSgenome.Hsapiens.UCSC.hg19, BSgenome.Hsapiens.UCSC.hg38,
        knitr, rmarkdown, ggplot2, testthat, VariantAnnotation
Description: Generate SuperSigs (supervised mutational signatures) from single nucleotide variants in the cancer genome. Functions included in the package allow the user to learn supervised mutational signatures from their data and apply them to new data. The methodology is based on the one described in Afsari (2021, ELife).
biocViews: FeatureExtraction, Classification, Regression, Sequencing,
        WholeGenome, SomaticMutation
BugReports: https://github.com/TomasettiLab/supersigs/issues
URL: https://tomasettilab.github.io/supersigs/
License: GPL-3
Encoding: UTF-8
LazyData: true
LazyDataCompression: gzip
RoxygenNote: 7.1.1
VignetteBuilder: knitr
Config/testthat/edition: 3
git_url: https://git.bioconductor.org/packages/supersigs
git_branch: devel
git_last_commit: 281d3c5
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.23
Date/Publication: 2025-10-31
NeedsCompilation: no
Packaged: 2025-11-01 00:31:34 UTC; biocbuild
Author: Albert Kuo [aut, cre] (ORCID: <https://orcid.org/0000-0001-5155-0748>),
  Yifan Zhang [aut],
  Bahman Afsari [aut],
  Cristian Tomasetti [aut]
Maintainer: Albert Kuo <albertkuo@jhu.edu>
