# Generated by roxygen2: do not edit by hand

export(get_signature)
export(make_matrix)
export(partial_signature)
export(predict_signature)
export(process_vcf)
export(simplify_signature)
import(assertthat)
import(caret)
import(dplyr)
import(rsample)
import(tidyr)
import(utils)
importFrom(Biostrings,getSeq)
importFrom(SummarizedExperiment,colData)
importFrom(SummarizedExperiment,rowRanges)
importFrom(SummarizedExperiment,seqinfo)
importFrom(methods,as)
importFrom(methods,new)
importFrom(rlang,.data)
importFrom(stats,binomial)
importFrom(stats,coef)
importFrom(stats,end)
importFrom(stats,glm)
importFrom(stats,median)
importFrom(stats,pbinom)
importFrom(stats,predict)
importFrom(stats,setNames)
importFrom(stats,start)
