#' Example dataset of mutations
#'
#' A dataset containing a list of mutations and other necessary attributes
#'
#' @format A data frame with 10 rows and 5 columns:
#' \describe{
#'   \item{sample_id}{ID of the patient}
#'   \item{age}{age of the patient}
#'   \item{chromosome}{chromosomal position of the mutation}
#'   \item{position}{position of the mutation}
#'   \item{ref}{original nucleotide}
#'   \item{alt}{mutated nucleotide}
#' }
"example_dt"